@@ -30,17 +30,27 @@ TestExpressionParser::TestExpressionParser()
 {
 }
 
+void TestExpressionParser::newClass_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("normalSyntax") << "<? class A { function __construct($a){} function foo() {} } $i = new A(1);";
+
+    QTest::newRow("trailingCommaInConstructor") << "<? class A { function __construct($a,){} function foo() {} } $i = new A(1);";
+
+    QTest::newRow("trailingCommaInInstantiation") << "<? class A { function __construct($a,){} function foo() {} } $i = new A(1,);";
+}
 
 void TestExpressionParser::newClass()
 {
-    //                 0         1         2         3         4         5         6         7
-    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
-    QByteArray method("<? class A { function foo() {} } $i = new A();");
+    QFETCH(QString, code);
 
-    TopDUContext* top = parse(method, DumpNone);
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
     DUChainReleaser releaseTop(top);
     DUChainWriteLocker lock(DUChain::lock());
 
+    QVERIFY(top->problems().isEmpty());
+
     ExpressionParser p(true);
     ExpressionEvaluationResult res = p.evaluateType(QByteArray("$i"), DUContextPointer(top), CursorInRevision(1, 0));
     QVERIFY(res.type());
@@ -230,6 +240,35 @@ void TestExpressionParser::globalFunction()
     QCOMPARE(res.allDeclarations().first().data(), top->localDeclarations().first());
 }
 
+void TestExpressionParser::globalFunctionCall_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("normalSyntax") << "<? function foo($a) {} foo(1);";
+
+    QTest::newRow("trailingComma") << "<? function foo($a) {} foo(1,);";
+}
+
+void TestExpressionParser::globalFunctionCall()
+{
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QFETCH(QString, code);
+
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock(DUChain::lock());
+
+    QVERIFY(top->problems().isEmpty());
+
+    ExpressionParser p(true);
+    ExpressionEvaluationResult res = p.evaluateType(QByteArray("foo"), DUContextPointer(top), CursorInRevision(1, 0));
+    QVERIFY(res.type());
+    QVERIFY(res.type().dynamicCast<FunctionType>());
+    QCOMPARE(res.allDeclarations().count(), 1);
+    QCOMPARE(res.allDeclarations().first().data(), top->localDeclarations().first());
+}
+
 void TestExpressionParser::chainCall()
 {
     //                 0         1         2         3         4         5         6         7
@@ -453,16 +492,27 @@ void TestExpressionParser::operations()
     QVERIFY(res.type().staticCast<IntegralType>()->dataType() == IntegralType::TypeInt);
 }
 
+void TestExpressionParser::findArg_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("normalSyntax") << "<? class A{} function foo($arg, &$bar, A &$a) {  } ";
+
+    QTest::newRow("trailingComma") << "<? class A{} function foo($arg, &$bar, A &$a,) {  } ";
+}
+
 void TestExpressionParser::findArg()
 {
     //                 0         1         2         3         4         5         6         7
     //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
-    QByteArray method("<? class A{} function foo($arg, &$bar, A &$a) {  } ");
+    QFETCH(QString, code);
 
-    TopDUContext* top = parse(method, DumpNone);
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
     DUChainReleaser releaseTop(top);
     DUChainWriteLocker lock(DUChain::lock());
 
+    QVERIFY(top->problems().isEmpty());
+
     ExpressionParser p(true);
 
     QCOMPARE(top->childContexts().size(), 3);
@@ -706,5 +756,99 @@ void TestExpressionParser::invalidArgumentUnpacking()
     QVERIFY(!top->problems().isEmpty());
 }
 
+void TestExpressionParser::closure_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("normalSyntax") << "<? $f = function($a, $b) { return $a + $b; }; \n";
+
+    QTest::newRow("trailingComma") << "<? $f = function($a, $b,) { return $a + $b; }; \n";
 }
 
+void TestExpressionParser::closure()
+{
+    QFETCH(QString, code);
+
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QVERIFY(top->problems().isEmpty());
+
+    ExpressionParser p(true);
+
+    QCOMPARE(top->childContexts().size(), 2);
+    QVERIFY(top->childContexts().at(0)->type() == DUContext::Function);
+    QVERIFY(top->childContexts().at(1)->type() == DUContext::Other);
+
+    ExpressionEvaluationResult res = p.evaluateType(QByteArray("$a"), DUContextPointer(top->childContexts().at(0)),
+                                                    CursorInRevision(0, 20));
+    QVERIFY(res.type().dynamicCast<IntegralType>());
+    QCOMPARE(res.type().staticCast<IntegralType>()->dataType(), static_cast<uint>(IntegralType::TypeMixed));
+
+    res = p.evaluateType(QByteArray("$b"), DUContextPointer(top->childContexts().at(0)),
+                         CursorInRevision(0, 20));
+    QVERIFY(res.type().dynamicCast<IntegralType>());
+    QCOMPARE(res.type().staticCast<IntegralType>()->dataType(), static_cast<uint>(IntegralType::TypeMixed));
+}
+
+void TestExpressionParser::closureUse_data()
+{
+    QTest::addColumn<QString>("code");
+
+    QTest::newRow("normalSyntax") << "<? $c = 'foo'; $f = function($a, $b) use ($c) { return $c . ':' . $a . '/' . $b; }; \n";
+
+    QTest::newRow("trailingCommaInParameterList") << "<? $c = 'foo'; $f = function($a, $b,) use ($c) { return $c . ':' . $a . '/' . $b; }; \n";
+
+    QTest::newRow("trailingCommaInUseList") << "<? $c = 'foo'; $f = function($a, $b) use ($c,) { return $c . ':' . $a . '/' . $b; }; \n";
+}
+
+void TestExpressionParser::closureUse()
+{
+    QFETCH(QString, code);
+
+    TopDUContext* top = parse(code.toUtf8(), DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QVERIFY(top->problems().isEmpty());
+
+    ExpressionParser p(true);
+
+    QCOMPARE(top->childContexts().size(), 3);
+    QVERIFY(top->childContexts().at(0)->type() == DUContext::Function);
+    QVERIFY(top->childContexts().at(1)->type() == DUContext::Other);
+    QVERIFY(top->childContexts().at(2)->type() == DUContext::Other);
+
+    ExpressionEvaluationResult res = p.evaluateType(QByteArray("$c"), DUContextPointer(top->childContexts().at(1)),
+                                                    CursorInRevision(0, 50));
+    QVERIFY(res.type().dynamicCast<IntegralType>());
+    QCOMPARE(res.type().staticCast<IntegralType>()->dataType(), static_cast<uint>(IntegralType::TypeString));
+
+    res = p.evaluateType(QByteArray("$a"), DUContextPointer(top->childContexts().at(0)),
+                                                    CursorInRevision(0, 34));
+    QVERIFY(res.type().dynamicCast<IntegralType>());
+    QCOMPARE(res.type().staticCast<IntegralType>()->dataType(), static_cast<uint>(IntegralType::TypeMixed));
+
+    res = p.evaluateType(QByteArray("$b"), DUContextPointer(top->childContexts().at(0)),
+                         CursorInRevision(0, 34));
+    QVERIFY(res.type().dynamicCast<IntegralType>());
+    QCOMPARE(res.type().staticCast<IntegralType>()->dataType(), static_cast<uint>(IntegralType::TypeMixed));
+}
+
+void TestExpressionParser::closureInvalidUse()
+{
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? $c = 'foo'; $f = function($a, $b) use () { return $c . ':' . $a . '/' . $b; }; \n");
+
+    TopDUContext* top = parse(method, DumpNone);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QVERIFY(!top->problems().isEmpty());
+}
+
+}
+
+#include "moc_expressionparser.cpp"
